# Test plan for OSM validation

[TOC]

## Pre-requirements

These tests assume that the VIM meets the following conditions:

- There must be a pre-provisioned VIM network, intended for VNF management, with DHCP enabled, reachable from OSM. Two options here:
  - A tenant network (e.g. `VIM_MGMT_NET` connected to an external network where floating IPs can be allocated. In this case, OSM will have to be informed about the use of floating IP in the VIM:

    ```bash
    osm vim-create ... --config '{use_floating_ip: True, management_network_name: VIM_MGMT_NET}'
    # The network vnf-mgmt will be the default management network if no mapping is specified at instantiation time
    ```

  - A provider network `VIM_MGMT_NET`, manually provisioned. In this case, the addition of the VIM will be as follows:

    ```bash
    osm vim-create ... --config '{management_network_name: VIM_MGMT_NET}'
    # The network vnf-mgmt will be the default management network if no mapping is specified at instantiation time
    ```

- If the VIM supports virtual device role tagging for proper interface identification, the following config parameter is expected:

  ```bash
  osm vim-create ... --config '{microversion: 2.32}'
  ```

- If the VIM supports multi-segment networks, the following config parameters are expected:

  ```bash
  osm vim-create ... --config '{multisegment_support: true, l2gw_support: true, multisegment_vlan_range: ["200-500"]}'
  ```

In addition, the following images are needed for the tests, and must be uploaded to the corresponding VIM before the test starts:

- `ubuntu18.04`: [bionic-server-cloudimg-amd64.img](https://cloud-images.ubuntu.com/bionic/current/bionic-server-cloudimg-amd64.img)
- `ubuntu20.04`: [focal-server-cloudimg-amd64.img](https://cloud-images.ubuntu.com/focal/current/focal-server-cloudimg-amd64.img)
- `cirros-0.3.5-x86_64-disk.img`: [cirros-0.3.5-x86_64-disk.img](http://download.cirros-cloud.net/0.3.5/cirros-0.3.5-x86_64-disk.img)
- `vyos-1.1.7`: [vyos-1.1.7-cloudinit.qcow2](https://osm-download.etsi.org/ftp/images/tests/vyos-1.1.7-cloudinit.qcow2)

In the case of OpenStack VIMs, they can be uploaded following this procedure:

```bash
DISK_FORMAT=qcow2
#DISK_FORMAT=vmdk   # For VIO, use this format instead of qcow2

# Test images
openstack image show cirros-0.3.5-x86_64-disk.img >/dev/null 2>&1 || \
wget http://download.cirros-cloud.net/0.3.5/cirros-0.3.5-x86_64-disk.img && \
openstack image create --disk-format $DISK_FORMAT --public --file cirros-0.3.5-x86_64-disk.img cirros-0.3.5-x86_64-disk.img && \
rm cirros-0.3.5-x86_64-disk.img

openstack image show ubuntu18.04 >/dev/null 2>&1 || \
wget https://cloud-images.ubuntu.com/bionic/current/bionic-server-cloudimg-amd64.img && \
openstack image create --disk-format $DISK_FORMAT --public --file bionic-server-cloudimg-amd64.img ubuntu18.04 && \
rm bionic-server-cloudimg-amd64.img

openstack image show ubuntu20.04 >/dev/null 2>&1 || \
wget  https://cloud-images.ubuntu.com/focal/current/focal-server-cloudimg-amd64.img&& \
openstack image create --disk-format $DISK_FORMAT --public --file focal-server-cloudimg-amd64.img ubuntu20.04 && \
rm focal-server-cloudimg-amd64.img

openstack image show vyos-1.1.7 >/dev/null 2>&1 || \
wget https://osm-download.etsi.org/ftp/images/tests/vyos-1.1.7-cloudinit.qcow2 && \
openstack image create --disk-format $DISK_FORMAT --public --file vyos-1.1.7-cloudinit.qcow2 vyos-1.1.7 && \
rm vyos-1.1.7-cloudinit.qcow2
```

## Basic tests

### \[BASIC-01\] CRUD operations on VIM targets

1. Create VIM

   ```bash
    osm vim-create --name <VIM_NAME> --account_type openstack --auth_url <URL> --user <USER> --password <PASSWORD> -- tenant <TENANT> --description "my description"
    # The config dictionary should have the relevant options for accessing the NB API via TLS
        --config '{...}'
    ```

2. Check metrics at Prometheus, visiting [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and looking for `osm_vim_status` metric.
   Validate that there is an entry corresponding to the `vim_account_id`.

3. Delete VIM

   ```bash
   osm vim-delete <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```

### \[BASIC-02\] Basic VNF

```bash
osm vnpkg-create hackfest_basic_vnf.tar.gz
osm nspkg-create hackfest_basic_ns.tar.gz
osm ns-create --ns_name hackfest1 --nsd_name hackfest_basic-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --ssh_keys <KEY1_PUBKEY_FILE> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hackfest1
osm vnf-list --ns hackfest1
ssh -i <KEY1_PRIVKEY_FILE> ubuntu@<MGMT_IP>
osm ns-delete hackfest1
```

### \[BASIC-03\] Multi-VDU VNF

```bash
osm vnpkg-create hackfest_multivdu_vnf.tar.gz
osm nspkg-create hackfest_multivdu_ns.tar.gz
osm ns-create --ns_name hackfest2 --nsd_name hackfest_multivdu-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hackfest2
# The management VMs, as declared in the NSD, have to be connected to VIM network "mgmt".
osm vnf-list --ns hackfest2
# Get some variables
MGMT_IP_VNF1=`osm vnf-list --ns hackfest2 |awk '$8=="1" {print $14}'`
MGMT_IP_VNF2=`osm vnf-list --ns hackfest2 |awk '$8=="2" {print $14}'`
# Then get the IP of the dataVM for each VNF in the following way
VNF1_ID=`osm vnf-list --ns hackfest2 |awk '$8=="1" {print $2}'`
osm vnf-show $VNF1_ID --literal |grep dataVM-eth0
dataVM_IP_VNF1=`osm vnf-show $VNF1_ID --literal |grep dataVM-eth0|awk '{print $3}' |awk -F "," '{print $1}'`
VNF2_ID=`osm vnf-list --ns hackfest2 |awk '$8=="2" {print $2}'`
osm vnf-show $VNF2_ID --literal |grep dataVM-eth0
dataVM_IP_VNF2=`osm vnf-show $VNF2_ID --literal |grep dataVM-eth0|awk '{print $3}' |awk -F "," '{print $1}'`
# SSH to mgmtVM of each VNF and get IP addresses
ssh osm@<MGMT_IP_VNF1> (pwd: osm4u)
ip a l
ssh osm@<MGMT_IP_VNF2> (pwd: osm4u)
ip a l
# Then ping from mgmtVM in VNF1 to mgmtVM in VNF2, and vice-versa with different packet sizes (64, 100, 1500, 9000)
# In mgmtVM of VNF1:
for i in 56 92 1492 8992; do ping -s $i -c 3 -W 1 <VNF2_MGMT_IP>; done
# In mgmtVM of VNF2:
for i in 56 92 1492 8992; do ping -s $i -c 3 -W 1 <VNF1_MGMT_IP>; done
# Jump to dataVM of each VNF from mgmtVM
ssh osm@<dataVM_IP_VNF1> (pwd: osm4u)
ip a l
ip a l
ssh osm@<dataVM_IP_VNF1> (pwd: osm4u)
# Then ping from dataVM in VNF1 to dataVM in VNF2, and vice-versa with different packet sizes (64, 100, 1500, 9000)
# In mgmtVM of VNF1:
for i in 56 92 1492 8992; do ping -s $i -c 3 -W 1 <VNF2_DATA_IP>; done
# In mgmtVM of VNF2:
for i in 56 92 1492 8992; do ping -s $i -c 3 -W 1 <VNF1_DATA_IP>; done
osm ns-delete hackfest2
```

In addition, to test that it is possible to specify management IP addresses as parameters at instantiation time:

```bash
osm ns-create --ns_name hackfest2 --nsd_name hackfest_multivdu-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>, vnfd-connection-point-ref: [ {member-vnf-index-ref: "1", vnfd-connection-point-ref: vnf-mgmt, ip-address: <IP address from your vnf management network> }]}]}'
osm ns-show hackfest2
osm vnf-list --ns hackfest2
# Get some variables
MGMT_IP_VNF1=`osm vnf-list --ns hackfest2 |awk '$8=="1" {print $14}'`
ssh osm@<MGMT_IP_VNF1> (pwd: osm4u)
ip a l
# Check that the IP address was the one assigned as instantiation parameter
```

### \[BASIC-04\] VNF with cloud-init

```bash
osm vnfpkg-create hackfest_cloudinit_vnf.tar.gz
osm nspkg-create hackfest_cloudinit_ns.tar.gz
osm ns-create --ns_name hackfest3 --nsd_name hackfest_cloudinit-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hackfest3
# The management VMs, as declared in the NSD, have to be connected to VIM network "mgmt".
osm vnf-list --ns hackfest3
ssh ubuntu@<MGMT_IP> (pwd: osm4u) # The password to the default user has been injected via cloud-init
sudo ls /root/helloworld.txt    # The file has been injected via cloud-init
sudo cat /root/helloworld.txt   # The file has been injected via cloud-init
osm ns-delete hackfest3
```

### \[BASIC-05\] Instantiation parameters in cloud-init

1. Onboard the packages

   ```bash
   osm vnfpkg-create ubuntu-cloudinit_vnf.tar.gz
   osm nspkg-create ubuntu-cloudinit_ns.tar.gz
   ```

2. Instantiate setting the new password

   ```bash
   osm ns-create --ns_name ubuntucloudinit --nsd_name ubuntu-cloudinit_nsd --vim_account <VIM_ACCOUNT_NAME> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ], additionalParamsForVnf: [ { member-vnf-index: "1", additionalParams: { password: "PASSWORD" } } ] }'
   ```

3. Check that we can access the VM with the password we set

   ```bash
   ssh ubuntu@<mgmt IP>
   ```

4. Delete the NS:

   ```bash
   osm ns-delete ubuntucloudinit
   ```

### \[BASIC-06\] VNF with charm

[VNFD Package](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ha_proxy_charm_vnf)

[NSD Package](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ha_proxy_charm_ns)

```bash
osm nfpkg-create ha_proxy_charm_vnf
osm nspkg-create ha_proxy_charm_ns
osm ns-create --ns_name basic-06-proxy-charm --nsd_name  ha_proxy_charm-ns  --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show basic-06-proxy-charm
osm vnf-list --ns basic-06-proxy-charm
# Connect to both VNFs and check that the file /home/ubuntu/first-touch exists
ssh ubuntu@<MGMT_IP1> (pwd: osm4u) # The password to the default user has been injected via cloud-init
ls /home/ubuntu/first-touch   #This file is created by the charm
ssh ubuntu@<MGMT_IP2> (pwd: osm4u) # The password to the default user has been injected via cloud-init
ls /home/ubuntu/first-touch   #This file is created by the charm
osm ns-action basic-06-proxy-charm --vnf_name 1 --action_name touch --params '{filename: /home/ubuntu/mytouch1}'
#Returns an operation ID
osm ns-op-list basic-06-proxy-charm
osm ns-op-show OP_ID1
# Wait until the operation is marked as COMPLETED.
osm ns-action basic-06-proxy-charm --vnf_name 2 --action_name touch --params '{filename: /home/ubuntu/mytouch2}'
# Returns an operation ID
osm ns-op-list basic-06-proxy-charm
osm ns-op-show OP_ID2
# Wait until the operation is marked as COMPLETED.
# There should be three operations (1 instantiate, 2 actions), all of them COMPLETED. Otherwise, test 045 failed.
# Connect to both VNFs and check that the files were created from the actions
ssh ubuntu@<MGMT_IP1> (pwd: osm4u) # The password to the default user has been injected via cloud-init
ls /home/ubuntu/mytouch1   #This file is created by the primitive
ssh ubuntu@<MGMT_IP2> (pwd: osm4u) # The password to the default user has been injected via cloud-init
ls /home/ubuntu/mytouch2   #This file is created by the primitive
osm ns-delete basic-06-proxy-charm
```

### \[BASIC-07\] Secure key management

1. Download and onboard the descriptors

   ```bash
   osm vnfpkg-create simple_nopasswd_vnf.tar.gz
   osm nspkg-create simple_nopasswd_ns.tar.gz
   ```

2. Instantiate the network service:

   ```bash
   osm ns-create --ns_name nopasswd --nsd_name simple-nopasswd-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [{name: mgmtnet, vim-network-name: <VIM_MGMT_NET>}]}'
   ```

3. Wait for the network service to be completed:

   ```bash
   osm ns-list
   ```

4. Verify the charm is not using a password:

   ```bash
   # Get juju status in the corresponding model for the NS, and get juju application name
   juju status -m <NS_ID>
   juju config -m <NS_ID> <juju app name>
   ```

   The variable "ssh-password" must be empty.

5. Verify the file `/home/ubuntu/first-touch` exists:

   ```bash
   osm vnf-list --ns nopasswd
   ssh ubuntu@<MGMT_IP1> (pwd: osm4u)
   ls /home/ubuntu/first-touch   # This file is created by the charm
   ```

6. Invoke the day-2 `touch` primitive:

   ```bash
   osm ns-action nopasswd --vnf_name 1 --action_name touch --params '{filename: /home/ubuntu/mytouch1}'
   osm ns-op-list nopasswd
   osm ns-op-show OP_ID1
   ```

7. Verify the file `/home/ubuntu/mytouch1` exists:

   ```bash
   ssh ubuntu@<MGMT_IP1>      # (pwd: osm4u)
   ls /home/ubuntu/mytouch1   # This file is created by ns-action
   ```

8. Delete the NS:

   ```bash
   osm ns-delete nopasswd
   ```

### \[BASIC-08\] Disable port security at network level

1. Create VIM

   ```bash
   osm vim-create --name <VIM_NAME> --account_type openstack --auth_url <URL> --user <USER> --password <PASSWORD> -- tenant <TENANT> --description "my description"
   With config dictionary: --config '{disable_network_port_security: True}'
   ```

2. Add packages

   ```bash
   osm vnfpkg-create hackfest_multivdu_vnf.tar.gz
   osm nspkg-create hackfest_multivdu_ns.tar.gz
   ```

3. Deploy in the VIM created in step 1

   ```bash
   osm ns-create --ns_name noPortSecurity --nsd_name hackfest_multivdu-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
   osm ns-list
   osm ns-show noPortSecurity
   osm ns-show noPortSecurity --literal
   # Get VIM network ID for datanet NS VLD
   osm ns-show noPortSecurity --literal |grep -A 1 noPortSecurity-datanet |grep vim-id
   # Check in openstack that the network is created with port security enabled as false
   opestack network show <vim_network_id>|grep port_security_enabled
   osm ns-delete noPortSecurity
   ```

### \[BASIC-09\] Manual VNF/VDU Scaling

This test can be run with any VNFD with a scaling-descriptor. You can use the descriptor from the autoscaling tests for this. Please note the "scaling-type" can be "manual" or "automatic" (which may produce an automatic scale-in right after a manual scale-out depending on the threshold and current metric value).

Onboard the descriptors:

```bash
osm vnfpkg-create hackfest_basic_metrics_vnf.tar.gz
osm nspkg-create hackfest_basic_metrics_ns.tar.gz
```

Launch the NS:

```bash
osm ns-create --ns_name hfmetrics --nsd_name hackfest_basic-ns-metrics --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hfmetrics
```

Scale the VNF:

```bash
osm vnf-scale --scale-out --scaling-group vdu_autoscale hfmetrics 1
```

Check that the SCALE operation has completed.

```bash
osm ns-op-list hfmetrics
```

Check that the VNF has two VDUs:

```bash
osm vnf-list --ns hfmetrics
osm vnf-show <VNF_ID>
```

### \[BASIC-10\] VNF SW upgrade

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Download and import the `hackfest3-mgmt` image
   ```bash
   wget https://osm-download.etsi.org/ftp/osm-3.0-three/1st-hackfest/images/hackfest3-mgmt.qcow2
   openstack image create \
      --disk-format qcow2 \
      --file hackfest3-mgmt.qcow2 \
      hackfest3-mgmt
   ```
2. Download and onboard the descriptors
   ```bash
   osm vnfd-create hackfest_simplecharm_vnf.tar.gz
   osm nsd-create hackfest_simplecharm_ns.tar.gz
   ```
3. Instantiate the network service:
   ```bash
   osm ns-create --ns_name simple --nsd_name hackfest-simplecharm-ns --vim_account    <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [{name: mgmtnet, vim-network-name: <MGMT    NETWORK>}]}'
   ```
4. Wait for the network service to be complete/complete
   ```bash
   osm ns-list
   ```
5. For each of the following day-2 primitives: start, stop, restart, upgrade:
   ```bash
   osm ns-action simple --vnf_name 1 --action_name <primitive> --params '{}' (Returns an operation    ID)
   ```
6. Verify operationState is COMPLETED
   ```bash
   osm ns-op-list simple
   osm ns-op-show <operational ID>
   ```

### \[BASIC-11\] Native charms

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Download and import the following packages:
   - [`VNFD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/native_charm_vnf)
   - [`NSD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/native_charm_ns)
2. Onboard the packages:
   ```bash
   osm nfpkg-create native_charm_vnf
   osm nspkg-create native_charm_ns
   ```
3. Instantiate the Network Service:
   ```bash
   osm ns-create --ns_name basic-11-native-charms --nsd_name native_charm-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```
5. Wait for ns state to become `READY`, and current operation to be `IDLE`.
6. Switch to the Juju model for this network service (the id of the ns instance via `osm ns-list`).
   ```bash
   juju switch ec73e48c-ed4d-45a3-9b8e-b5eee85bb01d
   ```
7. Run `juju status`:
   ```text
   Model                                 Controller  Cloud/Region       Version  SLA          Timestamp
   ec73e48c-ed4d-45a3-9b8e-b5eee85bb01d  controller  lxd-cloud/default  2.8.1    unsupported  12:22:35+02:00

   App                                     Version  Status  Scale  Charm          Store  Rev  OS      Notes
   app-vnf-28083fea5dad-vdu-mgmtvm-cnt-z0           active      1  simple-native  local    1  ubuntu  
   app-vnf-a1f49cf974ed-vdu-mgmtvm-cnt-z0           active      1  simple-native  local    0  ubuntu  

   Unit                                       Workload  Agent  Machine  Public address  Ports  Message
   app-vnf-28083fea5dad-vdu-mgmtvm-cnt-z0/0*  active    idle   1        172.16.100.31          
   app-vnf-a1f49cf974ed-vdu-mgmtvm-cnt-z0/0*  active    idle   0        172.16.100.117         

   Machine  State    DNS             Inst id                Series  AZ  Message
   0        started  172.16.100.117  manual:172.16.100.117  bionic      Manually provisioned machine
   1        started  172.16.100.31   manual:172.16.100.31   bionic      Manually provisioned machine
   ```
8. Verify that `app-vnf-28083fea5dad-vdu-mgmtvm-cnt-z0/0` and `app-vnf-a1f49cf974ed-vdu-mgmtvm-cnt-z0/0` units are active.
9. Verify that machine 0's and 1's instance id begins with `manual`.
10. List actions and check if day-1 operations have been executed:
   ```bash
   $ juju show-action-status
   actions:
   - action: touch
     completed at: "2020-07-14 10:18:58"
     id: "2"
     status: completed
     unit: app-vnf-a1f49cf974ed-vdu-mgmtvm-cnt-z0/0
   - action: touch
     completed at: "2020-07-14 10:19:05"
     id: "4"
     status: completed
     unit: app-vnf-28083fea5dad-vdu-mgmtvm-cnt-z0/0
   ```

### \[BASIC-12\] NS primitives

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Download and import the following packages:
   - [`Nscharm ns.tar.gz`](https://osm.etsi.org/wiki/index.php/File:Nscharm_ns.tar.gz)
   - [`Nscharm policy vnf.tar.gz`](https://osm.etsi.org/wiki/index.php/File:Nscharm_policy_vnf.tar.gz)
   - [`Nscharm user vnf.tar.gz`](https://osm.etsi.org/wiki/index.php/File:Nscharm_user_vnf.tar.gz)
2. Update the NS package
   - The ns package needs to be updated with the Juju credentials for your OSM installation.
     ```bash
     tar xf Nscharm_ns.tar.gz
     vi nscharm_ns/nscharm_nsd.yaml
     ```
   - Replace the value of `juju-password` with the one from your `~/.local/share/juju/accounts.yaml` and save:
     ```bash
     tar cfz Nscharm_ns.tar.gz nscharm_ns/
     ```
3. Onboard the packages:
   ```bash
   osm upload-package Nscharm_policy_vnf.tar.gz
   osm upload-package Nscharm_user_vnf.tar.gz
   osm upload-package Nscharm_ns.tar.gz
   ```
4. Instantiate the Network Service. Note: if you change the `ns_name`, you'll need to update the NSD's ns-configuration with the same name.
   ```bash
   osm ns-create --ns_name test_nscharm --nsd_name nscharm-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```
5. Wait for operational to become `running`, config status to be `configured`, and detailed status to be `done`.
6. Switch to the Juju model for this network service (the id of the ns instance via `osm ns-list`)
   ```bash
   juju switch 510086c6-99cb-466f-9214-e36f2f52c7ab
   ```
7. Check the status of primitives run by the initial-config-primitive:
   ```bash
   juju show-action-status
   ```
8. Validate that the actions were successful:
   ```yaml
   actions:
   - action: verify-ssh-credentials
     completed at: "2019-12-11 07:57:15"
     id: "1"
     status: completed
     unit: app-vnf-b24be8a7d707-vdu-uservm-cnt-z0/0
   - action: verify-ssh-credentials
     completed at: "2019-12-11 07:57:15"
     id: "2"
     status: completed
     unit: app-vnf-70e9ebb7ba72-vdu-policyvm-cnt-z0/0
   - action: add-user
     completed at: "2019-12-11 07:57:46"
     id: "3"
     status: completed
     unit: app-z/0
   - action: add-user
     completed at: "2019-12-11 07:57:25"
     id: "4"
     status: completed
     unit: app-vnf-b24be8a7d707-vdu-uservm-cnt-z0/0
   - action: set-policy
     completed at: "2019-12-11 07:57:38"
     id: "5"
     status: completed
     unit: app-vnf-70e9ebb7ba72-vdu-policyvm-cnt-z0/0
   ```

### \[BASIC-13\] NS relations

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Download the following packages:
   ```bash
   - [`VNFD Package 1`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ns_relations_requires_vnf)
   - [`VNFD Package 2`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ns_relations_provides_vnf)
   - [`NSD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ns_relations_ns)
   ```
2. Onboard the packages:
   ```bash
   osm nfpkg-create ns_relations_requires_vnf
   osm nfpkg-create ns_relations_provides_vnf
   osm nspkg-create ns_relations_ns
   ```
3. Instantiate the Network Service:
   ```bash
   osm ns-create --ns_name nsrelations --nsd_name ns_relations-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```
5. Wait for ns state to become `READY`, and current operation to be `IDLE`.
6. Switch to the Juju model for this network service (the id of the ns instance via `osm ns-list`):
   ```bash
   juju switch 1cc0f5d3-cb20-432a-bb20-82bd026f3a7a
   ```
7. Run `juju status --relations`
   ```text
   Model                                 Controller  Cloud/Region       Version  SLA          Timestamp
   1cc0f5d3-cb20-432a-bb20-82bd026f3a7a  controller  lxd-cloud/default  2.8.1    unsupported  12:42:46+02:00

   App                   Version  Status  Scale  Charm                   Store  Rev  OS      Notes
   app-vnf-0c7aa56eaa4b           active      1  simple-native-requires  local    0  ubuntu
   app-vnf-6ba0c9b229c5           active      1  simple-native-provides  local    0  ubuntu

   Unit                     Workload  Agent  Machine  Public address  Ports  Message
   app-vnf-0c7aa56eaa4b/0*  active    idle   1        172.16.100.69
   app-vnf-6ba0c9b229c5/0*  active    idle   0        172.16.100.182

   Machine  State    DNS             Inst id                Series  AZ  Message
   0        started  172.16.100.182  manual:172.16.100.182  bionic      Manually provisioned machine
   1        started  172.16.100.69   manual:172.16.100.69   bionic      Manually provisioned machine

   Relation provider               Requirer                        Interface  Type     Message
   app-vnf-6ba0c9b229c5:interface  app-vnf-0c7aa56eaa4b:interface  interface  regular  
   ```
8. Verify that the relation is listed.

### \[BASIC-14\] VNF relations

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Download the following packages:
   - [`VNFD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/vnf_relations_vnf)
   - [`NSD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/vnf_relations_ns)
2. Onboard the packages:
   ```bash
   osm nfpkg-create vnf_relations_vnf
   osm nspkg-create vnf_relations_ns
   ```
3. Instantiate the Network Service:
   ```bash
   osm ns-create --ns_name vnfrelations --nsd_name vnf_relations-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```
4. Wait for ns state to become `READY`, and current operation to be `IDLE`.
5. Switch to the Juju model for this network service (the id of the ns instance via `osm ns-list`):
   ```bash
   juju switch b5186d8f-d579-479d-89fd-0130a18c3c05
   ```
6. Run `juju status --relations`
   ```text
   Model                                 Controller  Cloud/Region       Version  SLA          Timestamp
   b5186d8f-d579-479d-89fd-0130a18c3c05  controller  lxd-cloud/default  2.8.1    unsupported  12:30:59+02:00

   App                                           Version  Status  Scale  Charm                   Store  Rev  OS      Notes
   app-vnf-8eb9743b0a39-vdu-ple-provides-cnt-z0           active      1  simple-native-provides  local    0  ubuntu
   app-vnf-8eb9743b0a39-vdu-ple-requires-cnt-z0           active      1  simple-native-requires  local    0  ubuntu

   Unit                                             Workload  Agent  Machine  Public address  Ports  Message
   app-vnf-8eb9743b0a39-vdu-ple-provides-cnt-z0/0*  active    idle   1        172.16.100.69
   app-vnf-8eb9743b0a39-vdu-ple-requires-cnt-z0/0*  active    idle   0        172.16.100.182

   Machine  State    DNS             Inst id                Series  AZ  Message
   0        started  172.16.100.182  manual:172.16.100.182  bionic      Manually provisioned machine
   1        started  172.16.100.69   manual:172.16.100.69   bionic      Manually provisioned machine

   Relation provider                                       Requirer                                                Interface  Type     Message
   app-vnf-8eb9743b0a39-vdu-ple-provides-cnt-z0:interface  app-vnf-8eb9743b0a39-vdu-ple-requires-cnt-z0:interface  interface  regular  
   ```
7. Verify that the relation is listed in the bottom of the status message.

### \[BASIC-15\] RBAC setup

#### 1. Preparation of the environment

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

Optional, for enabling Keystone backend (otherwise it will use internal backend): restart NBI docker with the configurations for using Keystone:

```bash
docker service update osm_nbi --force \
  --env-add OSMNBI_AUTHENTICATION_BACKEND=keystone \
  --env-add OSMNBI_AUTHENTICATION_AUTH_URL=keystone \
  --env-add OSMNBI_AUTHENTICATION_AUTH_PORT=5000 \
  --env-add OSMNBI_AUTHENTICATION_USER_DOMAIN_NAME=default \
  --env-add OSMNBI_AUTHENTICATION_PROJECT_DOMAIN_NAME=default \
  --env-add OSMNBI_AUTHENTICATION_SERVICE_USERNAME=nbi \
  --env-add OSMNBI_AUTHENTICATION_SERVICE_PROJECT=service
```

#### 2. User operations

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Create a user
   ```bash
   osm user-create test_user --password test_password
   ```
2. List users and verify that `test_user` is present:
   ```bash
   osm user-list
   ```
3. Check the information for `test_user` using username:
   ```bash
   osm user-show test_user
   ```
4. Check the information for `test_user` using user ID:
   ```bash
   osm user-show <test_user_ID>
   ```
5. Update `test_user` and verify the new information:
   ```bash
   osm user-update --set-project admin,system_admin test_user
   osm user-update --add-project-role 'admin,project_user' test_user
   osm user-update --add-project-role 'system_admin,project_user' test_user
   osm user-show test_user
   ```
6. Try to authenticate with the user and run an operation:
   ```bash
   export OSM_USER=test_user
   export OSM_PROJECT=admin
   export OSM_PASSWORD=test_password
   osm ns-list
   ```
7. Delete `test_user` and verify it has been deleted:
   ```bash
   osm user-delete test_user
   osm user-list
   ```

#### 3. Project operations

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Create `test_project`:
   ```bash
   osm project-create test_project
   ```
2. List projects:
   ```bash
   osm project-list
   ```
3. Show `test_project` information:
   ```bash
   osm project-show test_project
   ```
4. Show `test_project` information using project ID:
   ```bash
   osm project-show <test_project_ID>
   ```
5. Update `test_project` information:
   ```bash
   osm project-update --name project_test test_project
   osm project-list
   ```
6. Delete `project_test`:
   ```bash
   osm project-delete project_test
   osm project-list
   ```

#### 4. Role operations

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Create `test_role`:
   ```bash
   osm role-create test_role
   ```
2. List roles:
   ```bash
   osm role-list
   ```
3. Show `test_role` information:
   ```bash
   osm role-show test_role
   ```
4. Show `test_role` information using role ID:
   ```bash
   osm role-show <test_role_ID>
   ```
5. Update `test_project` information:
   ```bash
   osm role-update --add "vims: true" test_role
   osm role-show test_role
   ```
6. Delete `project_test`:
   ```bash
   osm role-delete test_role
   osm role-list
   ```

### \[BASIC-16\] RBAC enforcement

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

### \[BASIC-17\] K8s Proxy Charms

1. Download the following packages:
   - [`VNFD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/k8s_proxy_charm_vnf)
   - [`NSD Package`](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/k8s_proxy_charm_ns)
2. Onboard the packages:
   ```bash
   osm nfpkg-create k8s_proxy_charm_vnf
   osm nspkg-create k8s_proxy_charm_ns
   ```
3. Instantiate the Network Service:
   ```bash
   osm ns-create --ns_name basic-17-k8s-proxy-charm  --nsd_name k8s_proxy_charm-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
   ```
4. Wait for ns state to become `READY`, and current operation to be `IDLE`.
5. Switch to the Juju model for this network service (the id of the ns instance via `osm ns-list`), adding `-k8s` at the end:
   ```bash
   juju switch cb2918cd-e957-41f7-8e5a-f13b4e0d7fe4-k8s
   ```
6. Run `juju status`
   ```text
   Model                                     Controller  Cloud/Region        Version  SLA          Timestamp
   cb2918cd-e957-41f7-8e5a-f13b4e0d7fe4-k8s  controller  microk8s/localhost  2.8.1    unsupported  12:52:46+02:00

   App                   Version  Status  Scale  Charm             Store  Rev  OS          Address        Notes
   app-vnf-89990e3e7182           active      1  simple-k8s-proxy  local    1  kubernetes  10.152.183.46  
   app-vnf-c609c57ddf8b           active      1  simple-k8s-proxy  local    0  kubernetes  10.152.183.39  

   Unit                     Workload  Agent  Address     Ports  Message
   app-vnf-89990e3e7182/0*  active    idle   10.1.57.83         
   app-vnf-c609c57ddf8b/0*  active    idle   10.1.57.82     
   ```
7. Verify that the column `OS` in the App level shows `kubernetes`
8. Verify day-1 operations
   ```bash
   osm vnf-list --ns basic-17-k8s-proxy-charm
   # Connect to both VNFs and check that the file /home/ubuntu/first-touch exists
   ssh ubuntu@<MGMT_IP1> (pwd: osm4u) # The password to the default user has been injected via cloud-init
   ls /home/ubuntu/first-touch   #This file is created by the charm
   ssh ubuntu@<MGMT_IP2> (pwd: osm4u) # The password to the default user has been injected via cloud-init
   ls /home/ubuntu/first-touch   #This file is created by the charm
   ```
9. Execute day-2 operations
   ```bash
   osm ns-action basic-17-k8s-proxy-charm --vnf_name 1 --action_name touch --params '{filename: /home/ubuntu/mytouch1}'
   osm ns-action basic-17-k8s-proxy-charm --vnf_name 2 --action_name touch --params '{filename: /home/ubuntu/mytouch2}'
   ```
10. Verify day-2 operations
   ```bash
   ssh ubuntu@<MGMT_IP1> (pwd: osm4u) # The password to the default user has been injected via cloud-init
   ls /home/ubuntu/mytouch1   #This file is created by the charm
   ssh ubuntu@<MGMT_IP2> (pwd: osm4u) # The password to the default user has been injected via cloud-init
   ls /home/ubuntu/mytouch2   #This file is created by the charm
   ```

#### Visibility of packages and instances

1. Check you have a `test_user` and `test_project` from previous test.
2. Run the following commands to assign administrative roles to the `test_user` over the `test_project`:
   ```bash
   osm user-update --add-project-role 'admin,project_user' test_user
   osm user-update --add-project-role 'test_project,account_manager' test_user
   ```
3. Export the following variables to authenticate with the new user:
   ```bash
   export OSM_USERNAME=test_user
   export OSM_PASSWORD=test_password
   export OSM_PROJECT=test_project
   ```
4. Create a VIM and instantiate any NS/VNF and validate that packages and instances are independent between the test_project and the admin project:
   ```bash
   osm vim-create ...
   wget https://osm-download.etsi.org/ftp/osm-3.0-three/examples/cirros_2vnf_ns/cirros_2vnf_ns.tar.gz
   wget https://osm-download.etsi.org/ftp/osm-3.0-three/examples/cirros_2vnf_ns/cirros_vnf.tar.gz
   osm vnfd-create cirros_vnf.tar.gz
   osm nsd-create cirros_2vnf_ns.tar.gz
   osm ns-create --nsd_name cirros_2vnf_ns --ns_name  test --vim_account [ VIM_NAME ]
   ```

## Service Assurance

### \[SA-01\] VNF with VIM-based metrics

This test will launch a Network Service formed by a single VNF, consisting of 1 VDU, and CPU and memory metrics will be collected.

Please make sure:

1. Your VIM has an accessible management network (in this case called "<VIM_MGMT_NET>")
2. Your VIM has the appropriate Ubuntu image, in this case `ubuntu16.04` (based on Xenial).
3. You run the following command to match your VIM metrics telemetry system's granularity, if different than 300s (recommended for this example is 60s or Gnocchi's `high archive-policy`):

```bash
docker service update --env-add OS_DEFAULT_GRANULARITY=60 osm_mon
```

Onboard the descriptors:

```bash
osm vnfpkg-create hackfest_basic_metrics_vnf.tar.gz
osm nspkg-create hackfest_basic_metrics_ns.tar.gz
```

Launch the NS:

```bash
osm ns-create --ns_name hfmetrics --nsd_name hackfest_basic-ns-metrics --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hfmetrics
```

Then make the following checks:

1. To check metrics at Prometheus, visit [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and look for `osm_cpu_utilization` and `osm_average_memory_utilization` (initial values could take some some minutes depending on the Telemetry system's granularity)
2. To check metrics at Grafana, visit [`http://[OSM_IP]:3000`](http://[OSM_IP]:3000) (`admin` / `admin`), you will find the 'OSM Sample Dashboard' (the two top charts correspond to this example)

FIXME: OLD: Any of the VMs can be accessed through SSH to further monitor (with htop, for example), and there is an HAProxy UI at port [`http://[HAProxy_IP]:32700`](http://[HAProxy_IP]:32700) (all credentials are `ubuntu` / `osm2018`)

### \[SA-02\] VNF with VIM-based metrics and auto-scaling

Deploy again the same NS instance used in \[SA-01\] and then increase CPU by entering the following command in the VM (SSH credentials are `ubuntu` / `osm4u`):

```bash
yes > /dev/null &   # 4 or 5 times!
# This will stress CPU to 100% and trigger a scale-out operation in POL. In this test, scaling will go up to 1 VDU (limited in the descriptor)
# Another option to stress CPU is using Apache Benchmark tool `ab`
# sudo apt install apache2-utils
# ab -n 5000000 -c 2 http://[load-balancer-ip]/test.php
```

This should trigger the scaling of the instance.

### \[SA-03\] VNF with VNF-based metrics (VDU level)

This will launch a Network Service formed by a couple of Ubuntu-based VMs, one of them auto-scalable based on system's CPU usage (from `/proc/stat`). In this example, a proxy charm collects metrics directly from the first VDU.

Make sure:

1. Your VIM has an accessible management network (in this case called "<VIM_MGMT_NET>").
2. Your VIM has an Xenial Ubuntu image, in this case called `ubuntu`.

Then onboard the descriptors:

```bash
osm vnfpkg-create ubuntuvm_vnfvdumetric_autoscale_vnf.tar.gz
osm nspkg-create ubuntuvm_vnfvdumetric_autoscale_ns.tar.gz
```

Finally, launch the NS:

```bash
osm ns-create --ns_name ubuntu01_vdumetric --nsd_name ubuntuvm_vnfvdumetric_autoscale_ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'
```

Testing:

1. To ensure the VNF is collecting metrics, wait for the charm to be ready, plus ~5 minutes to receive the first metric, then run:

   ```bash
   # Get the NS ID for NS ubuntu01_vdumetric
   osm ns-list | grep ubuntu01_vdumetric
   # Get juju status in the corresponding model for the NS, and get juju application name
   juju status -m <NS_ID>
   # Get metric from the juju application in the corresponding model for the NS
   juju metrics -m <NS_ID> <juju app name> # please note that granularity is 5 min
   ```

2. To check metrics at Prometheus, visit [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and look for `osm_load_pct`, `osm_load` and `osm_users`
3. To check metrics at Grafana, just visit [`http://[OSM_IP]:3000`](http://[OSM_IP]:3000) (`admin` / `admin`),
   you will find a sample dashboard (the two bottom charts correspond to this example, you would have to add one for `osm_load_pct` or edit the existing ones)

Delete the NS:

```bash
osm ns-delete ubuntu01_vdumetric
```

### \[SA-04\] VNF with VNF-based metrics (VDU level) and auto-scaling

Deploy again the same NS instance used in \[SA-03\] and then increase CPU to autoscale the Ubuntu VM. For that purpose, ssh to VDU1 and run the following command (or similar, to stress CPU):

```bash
yes > /dev/null &
# This will take load_pct higher than 80.00, and make the UbuntuVM scale towards its limit (3 scaled instances, according to the descriptor)
```

This should trigger the scaling of the instance.

NOTE: VDU Metrics collection for scaled instances is not supported. Please note that there is a [bug](https://osm.etsi.org/bugzilla/show_bug.cgi?id=585) that will show metrics in the scaled instances, but they are just the same measure from the original one.

### \[SA-05\] VNF with VNF-based metrics (VNF level)

This will launch a Network Service formed by a couple of Ubuntu VMs, one of them auto-scalable based on system's CPU usage (from `/proc/stat`). In this example, a proxy charm collects metrics from the VNF, represented by the first VDU. Make sure:

1. Your VIM has an accessible management network (in this case called "<VIM_MGMT_NET>")
2. Your VIM has an Xenial Ubuntu image, in this case called 'ubuntu16.04'

Onboard the descriptors:

```bash
osm vnfpkg-create ubuntuvm_vnfmetric_autoscale_vnf.tar.gz
osm nspkg-create ubuntuvm_vnfmetric_autoscale_ns.tar.gz
```

Launch the NS:

```bash
osm ns-create --ns_name ubuntu01_vnfmetric --nsd_name ubuntuvm_vnfmetric_autoscale_ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'
```

Testing:

1. To ensure the VNF is collecting metrics, wait for the charm to be ready, plus ~5 minutes to receive the first metric, then run:

   ```bash
   # Get the NS ID for NS ubuntu01_vnfmetric
   osm ns-list | grep ubuntu01_vnfmetric
   # Get juju status in the corresponding model for the NS, and get juju application name
   juju status -m <NS_ID>
   juju -m <NS_ID> metrics <juju app name> # please note that granularity is 5 min
   ```

2. To check metrics at Prometheus, visit [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and look for `osm_load_pct`, `osm_load` and `osm_users`
3. To check metrics at Grafana, just visit [`http://[OSM_IP]:3000`](http://[OSM_IP]:3000) (`admin` / `admin`), you will find a sample dashboard (the two bottom charts correspond to this example, you would have to add one for `osm_load_pct` or edit the existing ones)

Delete the NS:

```bash
osm ns-delete ubuntu01_vnfmetric
```

### \[SA-06\] VNF with VNF-based metrics (VNF level) and auto-scaling

\[SA-05\] 
Deploy again the same NS instance used in \[SA-04\] and then increase the CPU utilization to autoscale the Ubuntu VM. For that purpose, ssh to VDU1 and run the following command to stress CPU:

```bash
osm vnf-list --ns ubuntu01_vnfmetric
ssh ubuntu@<MGMT_IP>    # [pwd: osm2018]
yes > /dev/null &
# This will take load_pct higher than 80.00, and make the UbuntuVM scale towards its limit (3 scaled instances, according to the descriptor)
```

This should trigger the scaling of the instance.

NOTE: VNF Metrics collection for scaled instances is not supported.

### \[SA-07\] Events or alarms coming from SA-related VNFs in the NS

1. Onboard the packages:

   ```bash
   osm vnfpkg-create cirros_vdu_alarm_vnf.tar.gz
   osm nspkg-create cirros_vdu_alarm_ns.tar.gz
   ```

2. Instantiate the NS:

   ```bash
   osm ns-create --ns_name cirros_alarm --nsd_name cirros_vdu_alarm_ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [{name: cirros_nsd_vld1, vim-network-name: <VIM_MGMT_NET>}]}'
   ```

3. Wait until metrics are collected. Validate this by checking Prometheus.

4. Check POL logs:

   ```bash
   docker logs $(docker ps -qf "name=osm_pol.1")
   ```

5. Validate notification request has been made. Example:

   ```text
   05/28/2019 04:29:07 PM - osm_policy_module.alarming.service - INFO - Executing request to url https://webhook.site/e65ad8a0-53b5-4d7f-bd50-c1a50b260871 for vnf alarm alarm-1 with status alarm
   ```

## Network Slicing

### \[SLICING-01\] Basic Network Slicing

1. Onboard the packages and the Network Slice Templates

   ```bash
   osm vnfd-create slice_basic_vnf
   osm vnfd-create slice_basic_middle_vnfd
   osm nsd-create slice_basic_ns
   osm nsd-create slice_basic_middle_nsd
   osm netslice-template-create slice_basic_nst/slice_basic_nst.yaml
   ```

2. Instantiate the Slice

   ```bash
   osm nsi-create --nsi_name slicebasic --nst_name slice_basic_nst --vim_account <VIM_TARGET> --config '{netslice-vld: [ {name: slice_vld_mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'
   # Wait for the slice to be configured
   osm nsi-list --filter name="slicebasic"
   ```
   
3. Get the slice Middle VNF IP address

   ```bash
   # Get the ns id of the middle ns (called slicebasic.slice_basic_nsd_2)
   osm ns-list | grep slicebasic.slice_basic_nsd_2 | awk '{print $4}'
   # Get the IP address of the first (and only) VNF in the ns: MGMT_IP
   osm vnf-list --filter member-vnf-index-ref=1 | grep <ns_id> | awk '{print $14}'
   ```

4. Test Ping to the middle VNF

   ```bash
   ping -c 5 -W 1 <MGMT_IP > /dev/null && echo OK
   ```

5. SSH to the middle VNF

   ```bash
   ssh osm@<MGMT_IP>  [pwd: osm4u]
   ```


6. Get all the IP addresses of the slice

   ```bash
   # Get the list of ns of the slice
   osm ns-list | grep slicebasic | awk '{print $4}'
   # For each of the ns in the list, get the list of the VNFs
   osm vnf-list | grep <ns_id> | awk '{print $2}'
   # For each VNFs in the list, get the list of the IPs
   osm vnf-show <vnd_id> --filter vdur --literal | grep -o '[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}' | sort -t: -u -k1,1
   ```

7. Test first slice connectivity: Ping all the VMs in the slice

   ```bash
   ssh osm@<MGMT_IP>  [pwd: osm4u]
   # For each of the IP addresses obtained in step #5
   ping -c 5 -W 1 <ip_address> > /dev/null && echo OK
   ```

8. Stop the first slice intance

   ```bash
   osm nsi-delete slicebasic
   ```

9. Delete the template and packages

   ```bash
   osm netslice-template-delete slice_basic_nst
   osm netslice-template-delete slice_basic_nst2
   osm nsd-delete slice_basic_nsd
   osm nsd-delete slice_basic_middle_nsd
   osm vnfd-delete slice_basic_vnfd
   osm vnfd-delete slice_basic_middle_vnfd
   ```

### \[SLICING-02\] Complex Network Slicing

1. Onboard the packages and the Network Slice Templates

   ```bash
   osm vnfd-create slice_basic_vnf
   osm vnfd-create slice_basic_middle_vnfd
   osm nsd-create slice_basic_ns
   osm nsd-create slice_basic_middle_nsd
   osm netslice-template-create slice_basic_nst/slice_basic_nst.yaml
   osm netslice-template-create slice_basic_nst/slice_basic_2nd_nst.yaml
   ```

2. Instantiate the first Slice

   ```bash
   osm nsi-create --nsi_name slicebasic --nst_name slice_basic_nst --vim_account <VIM_TARGET> --config '{netslice-vld: [ {name: slice_vld_mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'
   # Wait for the slice to be configured
   osm nsi-list --filter name="slicebasic"
   ```
   
3. Instantiate the second Slice

   ```bash
   osm nsi-create --nsi_name sliceshared --nst_name slice_basic_nst2 --vim_account <VIM_TARGET> --config '{netslice-vld: [ {name: slice_vld_mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'
   # Wait for the slice to be configured
   osm nsi-list --filter name="sliceshared"
   ```
   

4. Check that the Network services have been created.

   ```bash
   # Count the Network services in the first slice
   osm ns-list | grep slicebasic | wc -l
   # Count the Network services in the second slice
   osm ns-list | grep sliceshared | wc -l
   # The addition should be 4
   ```

4. Get the slice Middle VNF IP address

   ```bash
   # Get the ns id of the middle ns (called slicebasic.slice_basic_nsd_2)
   osm ns-list | grep slicebasic.slice_basic_nsd_2 | awk '{print $4}'
   # Get the IP address of the first (and only) VNF in the ns: MGMT_IP
   osm vnf-list --filter member-vnf-index-ref=1 | grep <ns_id> | awk '{print $14}'
   ```

5. Get all the IP addresses of the first slice

   ```bash
   # Get the list of ns of the slice
   osm ns-list | grep slicebasic | awk '{print $4}'
   # For each of the ns in the list, get the list of the VNFs
   osm vnf-list | grep <ns_id> | awk '{print $2}'
   # For each VNFs in the list, get the list of the IPs
   osm vnf-show <vnd_id> --filter vdur --literal | grep -o '[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}' | sort -t: -u -k1,1
   ```

6. Test Ping to the middle VNF

   ```bash
   ping -c 5 -W 1 <MGMT_IP > /dev/null && echo OK
   ```

7. SSH to the middle VNF

   ```bash
   ssh osm@<MGMT_IP>  [pwd: osm4u]
   ```

8. Test first slice connectivity: Ping all the VMs in the slice

   ```bash
   ssh osm@<MGMT_IP>  [pwd: osm4u]
   # For each of the IP addresses obtained in step #5
   ping -c 5 -W 1 <ip_address> > /dev/null && echo OK
   ```

9. Stop the first slice intance

   ```bash
   osm nsi-delete slicebasic
   ```

10. Count the Network services remaining after deleting the first slice

   ```bash
   # Count the Network services in the first slice
   osm ns-list | grep slicebasic | wc -l
   # Count the Network services in the second slice
   osm ns-list | grep sliceshared | wc -l
   # The addition should be 2
   ```

11. Get all the IP addresses of the second slice

   ```bash
   # Get the list of ns of the slice
   osm ns-list | grep sliceshared | awk '{print $4}'
   # For each of the ns in the list, get the list of the VNFs
   osm vnf-list | grep <ns_id> | awk '{print $2}'
   # For each VNFs in the list, get the list of the IPs
   osm vnf-show <vnd_id> --filter vdur --literal | grep -o '[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}' | sort -t: -u -k1,1
   ```

12. Test second slice connectivity: Ping all the VMs in the slice

   ```bash
   ssh osm@<MGMT_IP>  [pwd: osm4u]
   # For each of the IP addresses obtained in step #11
   ping -c 5 -W 1 <ip_address> > /dev/null && echo OK
   ```

13. Stop the second slice intance

   ```bash
   osm nsi-delete sliceshared
   ```

14. Delete the template and packages

   ```bash
   osm netslice-template-delete slice_basic_nst
   osm netslice-template-delete slice_basic_nst2
   osm nsd-delete slice_basic_nsd
   osm nsd-delete slice_basic_middle_nsd
   osm vnfd-delete slice_basic_vnfd
   osm vnfd-delete slice_basic_middle_vnfd
   ```

### \[SLICING-03\] Advanced Network Slicing

TODO: Under ellaboration. Similar to above, but with day-2 operations

## Kubernetes and cloud-native

### \[K8s-01\] Creation of Kubernetes cluster from OSM

A K8s cluster can be deployed using the following OSM packages:

- [k8s_jujucontroller_vnf](http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_jujucontroller_vnf.tar.gz)
- [k8s_jujumachine_vnf](http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_jujumachine_vnf.tar.gz)
- [k8s_juju](http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_juju_ns.tar.gz)

The NS (k8s_juju) consists of 1 deployer (k8s_jujucontroller_vnf) and 4 nodes (k8s_jujumachine_vnf) connected to a single network or vld (mgmtnet). The deployer is a Kubernetes installer based on Juju: it will configure the other 4 nodes to run a Kubernetes cluster. Behind the scenes, the deployer is a Juju controller where the 4 nodes are manually added to a Juju model, then a juju bundle is deployed on that model.

The following figure illustrates the NS.

![k8s-cluster-with-osm-packages](assets/800px-k8s-cluster-with-osm-packages.png)

The steps to be done are the following:

1. Onboard the packages

   ```bash
   wget http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_jujucontroller_vnf.tar.gz
   wget http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_jujumachine_vnf.tar.gz
   wget http://osm-download.etsi.org/ftp/Packages/hackfests/k8s_juju_ns.tar.gz
   osm nfpkg-create k8s_jujumachine_vnf.tar.gz
   osm nfpkg-create k8s_jujucontroller_vnf.tar.gz
   osm nspkg-create k8s_juju_ns.tar.gz
   ```

2. Instantiation parameters are controlled by `config.yaml`. The relevant parameters are the network in the VIM where the K8s API will eb exposed and where all nodes will be connected (`mgmt` in the example below) and the IP addresses from that network to be assigned to each machine (in the example below, `192.168.0.X`). Create `config.yaml`as follows:

   ```yaml
   ---
   additionalParamsForVnf:
     -
       member-vnf-index: k8s_juju
       additionalParams:
           MACHINE1: "192.168.0.161"
           MACHINE2: "192.168.0.162"
           MACHINE3: "192.168.0.163"
           MACHINE4: "192.168.0.164"
           MACHINE5: ""
           MACHINE6: ""
           MACHINE7: ""
           MACHINE8: ""
           MACHINE9: ""
           MACHINE10: ""
           BUNDLE: ""
   vld:
     -
       name: mgmtnet
       vim-network-name: mgmt              #The network in the VIM to connect all nodes of the clusters
       vnfd-connection-point-ref:
         -
           ip-address: "192.168.0.161"
           member-vnf-index-ref: k8s_vnf1
           vnfd-connection-point-ref: mgmt
         -
           ip-address: "192.168.0.162"
           member-vnf-index-ref: k8s_vnf2
           vnfd-connection-point-ref: mgmt
         -
           ip-address: "192.168.0.163"
           member-vnf-index-ref: k8s_vnf3
           vnfd-connection-point-ref: mgmt
         -
           ip-address: "192.168.0.164"
           member-vnf-index-ref: k8s_vnf4
           vnfd-connection-point-ref: mgmt
         -
           ip-address: "192.168.0.170"
           member-vnf-index-ref: k8s_juju
           vnfd-connection-point-ref: mgmt
   ```

3. Instantiate the NS

   ```bash
   osm ns-create --ns_name k8s-cluster --nsd_name k8s_juju --vim_account <VIM_ACCOUNT> --config_file config.yaml --ssh_keys ${HOME}/.ssh/id_rsa.pub
   ```

4. Connect to the machine running juju (`k8s_juju` in the NS), check that kubeconfig file exists and test kubectl:

   ```bash
   osm vnf-list --ns k8s-cluster --filter vnfd-ref=k8s_jujucontroller_vnf
   ssh ubuntu@<JUJU_CONTROLLER_IP_ADDRESS>
   cat .kube/config
   kubectl get all
   ```

### \[K8s-02\] Register and de-register Kubernetes clusters in OSM

1. Register the Kubernetes cluster in OSM

   ```bash
   osm k8scluster-add --creds <KUBECONFIG_FILE> --version v1 --vim <VIM_NAME> --k8s-nets '{"net1": <VIM_MGMT_NETWORK>}' --description "K8s cluster" osm-k8s-cluster
   ```

2. Check that the Kubernetes cluster is successully added

   ```bash
   # Get all k8sclusters. The Operational State of the cluster should be "ENABLED"
   osm k8scluster-list
   ```

3. De-register the Kubernetes cluster

   ```bash
   osm k8scluster-delete osm-k8s-cluster
   ```

4. Review that the cluster has been deleted

   ```bash
   # Get all k8sclusters. Our cluster should not appear.
   osm k8scluster-list
   ```

### \[K8s-03\] Deployment of CNF with Juju Bundle from OSM

1. Register the Kubernetes cluster in OSM:

   ```bash
   osm k8scluster-add --creds <KUBECONFIG_FILE> --version v1 --vim <VIM_NAME> --k8s-nets '{"net1": <VIM_MGMT_NETWORK>}' --description "K8s cluster" osm-k8s-cluster
   ```

2. Check that the Kubernetes cluster is successully added:

   ```bash
   # Get all k8sclusters. The Operational State of the cluster should be "ENABLED"
   osm k8scluster-list
   ```

3. Onboard the packages:

   ```bash
   osm nfpkg-create hackfest_simple_k8s_vnfd.tar.gz
   osm nspkg-create hackfest_simple_k8s_nsd.tar.gz
   ```

4. Instantiate the Network Service:

   ```bash
   osm ns-create --ns_name simple-k8s --nsd_name  --vim_account <VIM_ACCOUNT_NAME>  --ssh_keys <PUBLIC_KEY> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
   ```

5. Wait the Network Service to be completed:

   ```bash
   osm ns-list
   ```

6. Delete the Network Service:

   ```bash
   osm ns-delete simple-k8s
   ```

7. Delete the descriptors:

   ```bash
   osm nspkg-delete hackfest-simple-k8s-ns
   osm nfpkg-delete hackfest-simple-k8s-vnf
   ```

8. If applicable, deregister the Kubernetes cluster from OSM:

   ```bash
   osm k8scluster-delete osm-k8s-cluster
   ```

### \[K8s-04\] Deployment of OpenLDAP CNF with Helm Chart from OSM

1. Register the Kubernetes cluster in OSM:

   ```bash
   osm k8scluster-add --creds <KUBECONFIG_FILE> --version v1 --vim <VIM_NAME> --k8s-nets '{"net1": <VIM_MGMT_NETWORK>}' --description "K8s cluster" osm-k8s-cluster
   ```

2. Check that the Kubernetes cluster is successully added:

   ```bash
   # Get all k8sclusters. The Operational State of the cluster should be "ENABLED"
   osm k8scluster-list
   ```

3. Onboard the packages (from OSM's Packages Git repo):

   ```bash
   # If not previusly done:
   # $ git clone ssh://git@osm.etsi.org:29419/vnf-onboarding/osm-packages.git
   # $ cd osm-packages

   osm nfpkg-create openldap_knf
   osm nspkg-create openldap_ns
   ```

4. Instantiate the Network Service:

   ```bash
   osm ns-create --ns_name ldap --nsd_name openldap_ns --vim_account <VIM_ACCOUNT_NAME>  --ssh_keys <PUBLIC_KEY> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
   ```

5. Wait the Network Service to be completed:

   ```bash
   osm ns-list
   ```

6. Delete the Network Service:

   ```bash
   osm ns-delete ldap
   ```

7. Delete the descriptors:

   ```bash
   osm nspkg-delete openldap_ns
   osm nfpkg-delete openldap_knf
   ```

8. If applicable, deregister the Kubernetes cluster from OSM:

   ```bash
   osm k8scluster-delete osm-k8s-cluster
   ```

## EPA and performance

### \[EPA-01\] EPA+SR-IOV without underlay

As a precondition for this test, please make sure:

1. Your VIM has an accessible management network (in this case called "<VIM_MGMT_NET>")
2. Your VIM has the appropriate test image, in this case `ubuntu18.04`.
3. Your VIM uses a phuysical dataplane network (for instance, it was created with `dataplane_physical_net: physnet2` in its configuration).

Onboard the following packages on OSM ([refer to this repo for OSM packages](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages)):

```bash
osm nfpkg-create hackfest_basic_sriov_vnf
osm nspkg-create hackfest_basic_sriov_ns
```

Instantiate the NS:

```bash
osm ns-create --ns_name epa_01-epa_sriov_test --nsd_name hackfest_basic_sriov-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --ssh_keys <KEY1_PUBKEY_FILE> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
```

Wait for the VM instante to be up and SSH into it (credentials are ubuntu/osm4u).

After that, check an Ethernet device is enabled on the VM interfaces (you should see it as "Ethernet controller" istead of "Virtio" on the devices list):

```bash
lspci | grep "Ethernet controller" | grep -v "Virtio"
```

Finally, delete the NS and the onboarded packages:

```bash
osm ns-delete epa_01-epa_sriov_test
osm nsd-delete hackfest_basic_sriov-ns
osm vnfd-delete hackfest_basic_sriov-vnf
```

### \[EPA-02\] Support for additional EPA capabilities

Download and import the packages below:

```bash
osm nfpkg-create epa_quota_vnf.tar.gz
osm nspkg-create epa_quota_ns.tar.gz
```

Instantiate NS, check the flavor in VIM and delete NS:

```bash
osm ns-create --ns_name epa_quota --nsd_name epa_quota-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --ssh_keys <KEY1_PUBKEY_FILE> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show epa_quota
osm vnf-list --ns epa_quota
osm vnf-show <VNF_ID> --literal
# Get VIM_VM_ID
osm vnf-show <VNF_ID> --literal |grep vim-id
# Check the corresponding flavor created in open-stack, check that flavor metadata has the quota properties set as detailed in descriptor
openstack server show <VIM_VM_ID>
openstack flavor show <flavor_id> |grep properties
| properties                 | quota:cpu_reservation='512', quota:cpu_shares_level='custom', quota:cpu_shares_share='3000', quota:disk_io_shares_level='custom', quota:disk_io_shares_share='1000', quota:memory_limit='2048', quota:memory_reservation='512', quota:memory_shares_level='custom', quota:memory_shares_share='20480' |
# Delete NS
osm ns-delete epa_quota
```

### \[EPA-03\] CRUD operations on SDNC accounts

To be performed only SDN assist is required and an SDNC is available.

1. Create SDNC

   ```bash
   osm sdnc-create --name <SDNC_NAME> --type onos --ip-address <IP> --user <USER> --password <PASSWORD> --port <port> --switch_dpid <DPID>
   ```

2. Check metrics at Prometheus, visiting [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and looking for `osm_sdnc_status` metric. Validate that there is an entry corresponding to the registered sdnc.

3. Delete SDNC

   ```bash
   osm sdnc-delete <SDNC_NAME>|<SDNC_ID>
   ```

### \[EPA-04\] EPA underlay SR-IOV

```bash
osm vnfpkg-create hackfest_epasriov_vnf.tar.gz
osm nspkg-create hackfest_epasriov_ns.tar.gz
osm ns-create --ns_name hackfest4 --nsd_name hackfest_epasriov-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
osm ns-list
osm ns-show hackfest4
# The management VMs, as declared in the NSD, have to be connected to VIM network "<VIM_MGMT_NET>".
# Get the management IP address of the VNF in this way:
MGMT_IP_VNF1=`osm vnf-list --ns hackfest4 |awk '$8=="1" {print $14}'`
MGMT_IP_VNF2=`osm vnf-list --ns hackfest4 |awk '$8=="2" {print $14}'`
# Then get the IP of the dataVM for each VNF in the following way
VNF1_ID=`osm vnf-list --ns hackfest4 |awk '$8=="1" {print $2}'`
osm vnf-show $VNF1_ID --literal |grep eth0|grep internal
dataVM_IP_VNF1=`osm vnf-show $VNF1_ID --literal |grep eth0|grep internal|awk '{print $3}' |awk -F "," '{print $1}'`
VNF2_ID=`osm vnf-list --ns hackfest4 |awk '$8=="2" {print $2}'`
osm vnf-show $VNF2_ID --literal |grep eth0|grep internal
dataVM_IP_VNF2=`osm vnf-show $VNF2_ID --literal |grep eth0|grep internal|awk '{print $3}' |awk -F "," '{print $1}'`
#Connect to the mgmt VM of VNF 1
ssh ubuntu@<MGMT_IP_VNF1> (pwd: osm4u) # The password to the default user has been injected via cloud-init
ssh -i test4.pem ubuntu@<dataVM_IP_VNF1>  #Jump to the dataVM
# Do the same in VNF 2 and try to run pktgen using instructions in the next step
```

Get the port information from the VIM, required to get the MAC address if the NICs are not Niantic:

```bash
$ openstack port list --server hackfest4-1-dataVM-1
+--------------------------------------+------+---------------------------------------------------------------------------------------------------+--------+
| ID                                   | Name | MAC Address       | Fixed IP ddresses                                                             | Status |
+--------------------------------------+------+---------------------------------------------------------------------------------------------------+--------+
| 316df561-3cf3-4e76-b1fa-d2e221f636c6 | xe0  | fa:16:3e:d0:8c:c3 | ip_address='192.168.152.189', ubnet_id='6cca9a17-9af1-4e07-b08f-a65ccb707ef6' | ACTIVE |
| d41568c8-2cc1-487c-b20f-2ba7918534a5 | eth0 | fa:16:3e:31:42:91 | ip_address='192.168.157.13', ubnet_id='fddef527-1ff8-47d1-bf83-22243064808b'  | ACTIVE |
+--------------------------------------+------+---------------------------------------------------------------------------------------------------+--------+
$ openstack port list --server hackfest4-2-dataVM-1
+--------------------------------------+------+--------------------------------------------------------------------------------------------------+--------+
| ID                                   | Name | MAC Address       | Fixed IP ddresses                                                            | Status |
+--------------------------------------+------+--------------------------------------------------------------------------------------------------+--------+
| a9ad3bf9-1cc6-4650-860a-14cc0282d10a | eth0 | fa:16:3e:92:39:03 | ip_address='192.168.29.49', ubnet_id='7bec52f5-f256-4802-9164-8f0036783f60'  | ACTIVE |
| e20592e6-5f0b-42da-b8f0-948b77457682 | xe0  | fa:16:3e:b8:e3:25 | ip_address='192.168.152.56', ubnet_id='6cca9a17-9af1-4e07-b08f-a65ccb707ef6' | ACTIVE |
+--------------------------------------+------+--------------------------------------------------------------------------------------------------+--------+
```

Next, traffic must be sent from one VNF to the other. This procedure is documented in this presentation: <http://osm-download.etsi.org/ftp/osm-5.0-five/6th-hackfest/presentations/6th%20OSM%20Hackfest%20-%20Session%205%20-%20Modeling%20EPA%20capabilities%20in%20VNF.pdf>

If no SSH access to the dataVM, most probably the VIM metadata service doesn't provide the required information for virtual-device-role-tagging. In that case, access to the VM will happen via console and interfaces in the VM will have to be configured manually

```bash
# Disable udev metadata service (pktgen image runs this service to rename interfaces properly)
sudo systemctl disable udev-metadata.service
# Identify the PCI addresses of the network interfaces
lspci
# Include manually the mapping from PCI address to interface name
sudo vi /etc/udev/rules.d/70-persistent-net.rules
  ACTION=="add", SUBSYSTEM=="net", KERNELS=="0000:0b:00.0", NAME="eth0"
  ACTION=="add", SUBSYSTEM=="net", KERNELS=="0000:03:00.0", NAME="xe0"
# Edit network interfaces to enable eth0 via dhcp
sudo vi /etc/network/interfaces
sudo reboot
```

Check that the info from the metadata service is correct:

```bash
lspci
wget http://169.254.169.254/openstack/latest/meta_data.json
```

Check the CPU layout in the dataVM:

```bash
# Configure interfaces to use DPDK
cd /opt/dpdk-17.11
sudo -E ./usertools/cpu_layout.sh
```

```bash
# Configure interfaces to use DPDK
cd /opt/dpdk-17.11
sudo -E ./usertools/dpdk-devbind.py --status
sudo -E ./usertools/dpdk-devbind.py --bind=igb_uio xe0
# Run pktgen
cd /opt/pktgen-3.4.5/
#sudo -E ./app/x86_64-native-linuxapp-gcc/pktgen -n 3 -l 2-7 -- -P -m "[4-7].0"
sudo -E ./app/x86_64-native-linuxapp-gcc/pktgen -n 3 -l 2-7 -- -P -m "[4-5:6-7].0"
# With some NICs, pktgen mmight throw this error:
# i40evf_dev_configure(): VF can't disable HW CRC Strip
# In that case the command below aplies
#sudo -E ./app/x86_64-native-linuxapp-gcc/pktgen -n 3 -l 2-7 -- -P -m "[4-7].0" --crc-strip
sudo -E ./app/x86_64-native-linuxapp-gcc/pktgen -n 3 -l 2-7 -- -P -m "[4-5:6-7].0" --crc-strip
# From pktgen console, set the destination MAC address to the one used by xe0 interface in the other VNF, and send traffic
# Set Destination MAC where to send traffic
set all dst mac xx:xx:xx:xx:xx:xx
# Start traffic
start all
# Stop traffic
stop all
```

The expected throughtput in Mpps is 14.8 for 10G ports. For VF in 40G ports, the expected throughput is higher (~19-20 Mpps), but limited by the number of queues per VF (4). Check this link: <https://dpdk.readthedocs.io/en/v16.07/nics/intel_vf.html>

Finally, delete the NS:

```bash
osm ns-delete hackfest4
```

### \[EPA-05\] EPA underlay Passthrough

TODO: Similar to test case below, but with a package with Passhthough interfaces.

### \[PER-01\] Performance evaluation with vBNG and traffic generator

With the help of OSM, the following network service is deployed in the cloud:

![NS for testing NFV workloads](assets/NS-for-testing-NFV-workloads.png)

It consists of 2 single-VM VNFs, each with 34 vCPUs (pinned to 34 HW threads in 17 cores of the same socket),
32 GB RAM and 8 10G ports (in Passthrough).

Both have the tool "PROX" **\[2\]** **\[3\]**, which is an open-source tool, suitable to model and test
I/O intensive network applications making use of DPDK library. In this case, one of the VNFs had PROX
configured to work as a simple traffic generator, the other VNF had prox configured to work as a BNG facing
QinQ traffic from the metro and Internet traffic from the core, dealing appropriately wih the header
encapsulation/de-encapsulation, the routing, and a simple form of hierarchical QoS.

#### Overview

Once deployed, full line rate traffic is injected in both directions, and the bottleneck is measured in
the BNG interfaces, thus providing a figure of the supported I/O performance.

In order to confirm that the VMs were properly deployed, a double check is done through
Openstack command line client and through libvirt commands in the compute nodes where the VMs were deployed.

Then, while full line rate traffic is injected, we also inject traffic in the virtio interfaces to check
whether the traffic in virtio interfaces lead to an increase in the packet losses due to VM context switching.

Finally, we tear down the NS in OSM and check that the interfaces can be seen and are up in the compute node.

#### Step-by-step procedure

```bash
# Upload images to Openstack as described in the pre-requirements

# Add VIM and SDNC
osm vim-create ...
osm sdnc-create ...

# Onboard VNF and NS packages
osm nfpkg-create vbng34_vnf.tar.gz
osm nfpkg-create vbng50_vnf.tar.gz
osm nfpkg-create vbng_gen_vnf.tar.gz
osm nspkg-create vbng_test34_ns.tar.gz
osm nspkg-create vbng_test50_ns.tar.gz
osm nspkg-create vbng_6p_ns.tar.gz

# Instantiation
osm ns-create --ns_name testEPA --nsd_name vbng_test34 --vim_account XXXXXXX --config '{vld: [ {name: mgmt, vim-network-name: <VIM_MGMT_NET>} ] }'

# Get IP addresses of Generator and vBNG
osm vnf-list |grep vbng

# Run performance test
# Connect to vBNG
ssh tester@<IP1> [pwd:tester]
sudo su
# Check CPU layout: it should be 1 socket, 17 cores, 2 HW threads per core
/home/tester/dpdk-17.02/usertools/cpu_layout.py
# Run Prox
. ./set_dpdk_vars.sh
./dpdk.sh
cd prox
#./build/prox -f tid/broadwell-bng-4ports-qos-10-cores.cfg
./build/prox -f tid/broadwell-bng-8ports-qos-17-cores.cfg
# Press 2 to see only the port traffic
# Connect to Generator
ssh tester@<IP2> [pwd:tester]
sudo su
# Check CPU layout: it should be 1 socket, 17 cores, 2 HW threads per core
/home/tester/dpdk-17.02/usertools/cpu_layout.py
# Run Prox
. ./set_dpdk_vars.sh
./dpdk.sh
cd prox
#./build/prox -f tid/broadwell-gen-4ports.cfg
./build/prox -f tid/broadwell-gen-8ports.cfg
# Press 2 to see only the port traffic
# Capture the throughput of vBNG in Mpps

# Clean-up
osm ns-delete bng-bwl
osm ns-delete bng-sky
osm nspkg-delete vbng_test34
osm nspkg-delete vbng_test50
osm nfpkg-delete vbng34_vnf
osm nfpkg-delete vbng50_vnf
osm nfpkg-create vbng_gen_vnf
```

Before the clean-up, we will check that the flavor in Openstack has the following properties:

- `hw:cpu_policy='dedicated'`
- `hw:cpu_sockets='1'`
- `hw:cpu_thread_policy='require'`
- `hw:mem_page_size='large'`
- `hw:numa_mempolicy='strict'`
- `hw:numa_nodes='1'`

```bash
openstack server show testEPA-1-vbng34-VM-1 |egrep "flavor"
| flavor                              | testEPA-VM-flv-2 (33cd9a7f-6733-45b5-a559-4dc1f3cfdf95)  |
openstack server show testEPA-2-vbng_gen-VM-1 |egrep "flavor"
| flavor                              | testEPA-VM-flv-2 (33cd9a7f-6733-45b5-a559-4dc1f3cfdf95)  |
openstack flavor show 33cd9a7f-6733-45b5-a559-4dc1f3cfdf95
```

Then, we connect via ssh to the compute nodes and check the XML libvirt domains of the VMs.

- Memory: 32 GB backed with hugepages
- CPU: 34 CPU with dedicated CPU pininng
- NUMA pinning: memory should be allocated from the same NUMA node of the physical CPUs
- Virtual CPU topology: it should be coherent with the physical CPU topology: 17 cores, with
  2 HW threads per core, all in a single socket.
- PCI passthrough interfaces: they are identified by the source PCI address in libvirt XML domain
  and they should be in the same NUMA node of the physical CPUs and memory.

To get the hosts where each VM is running:

```bash
openstack hypervisor list
openstack server show testEPA-1-vbng34-VM-1 |grep "OS-EXT-SRV-ATTR:host"
openstack server show testEPA-2-vbng_gen-VM-1 |grep "OS-EXT-SRV-ATTR:host"
```

Then, on each host:

```bash
sudo virsh list
sudo virsh dumpxml <instance-id> |less
```

In the compute, we check that the source PCI address is in the same NUMA node of the physical CPUs and memory.

Finally, we log into the VM itself and check that the configuration is correct.

```bash
sudo /home/tester/dpdk-17.02/usertools/cpu_layout.py
lspci |grep -i ether
```

Once we tear down the NS, we check in the compute nodes that the interfaces, previously assigned in Passthrough,
are up again.

```bash
sudo ip address list
```

## Platform resiliency and recovery

### [\RESILIENCY-01\] OSM platform resiliency

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

In case of installation based on Docker Swarm:

1. Check that OSM is working:
   ```bash
   osm vnfpkg-list
   osm vim-list
   ```
2. Check that all services are up and running (all replicas should be 1/1):
   ```bash
   docker service ls
   ```
3. Kill abruptly all docker containers:
   ```bash
   for i in {osm_keystone.1,osm_keystone-db.1,osm_lcm.1,osm_light-ui.1,osm_mon.1,osm_mongo.1,osm_nbi.1,osm_pol.1,osm_prometheus.1,osm_ro.1,osm_ro-db.1}; do
     docker rm -f $(docker ps |grep -i $i|awk '{print $1}')
   done
   ```
4. Check that all services recover (all replicas should be `1/1` after enough time ~5 minutes):
   ```bash
   docker service ls
   ```
5. Check that OSM is working again:
   ```bash
   osm vnfpkg-list
   osm vim-list
   ```

In case of installation based on K8s, the command to kill abruptly all containers (step 3) should be:

```bash
for i in {k8s_keystone,k8s_mysql,k8s_lcm,k8s_light-ui,k8s_mon_,k8s_mongo,k8s_nbi,k8s_pol,k8s_prometheus,k8s_ro}; do
  docker rm -f $(docker ps |grep -i $i|awk '{print $1}')
done
```

### [\RESILIENCY-02\] OSM platform recovery

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

In case of installation based on Docker Swarm:

1. Check that OSM is working:
   ```bash
   osm vnfpkg-list
   osm vim-list
   ```
2. Check that all services are up and running (all replicas should be `1/1`):
   ```bash
   docker service ls
   ```
3. Kill abruptly all docker containers:
   ```bash
   for i in {osm_kafka.1,osm_keystone.1,osm_keystone-db.1,osm_lcm.1,osm_light-ui.1,osm_mon.1,osm_mongo.1,osm_nbi.1,osm_pol.1,osm_prometheus.1,osm_ro.1,osm_ro-db.1,osm_zookeeper.1}; do
     docker rm -f $(docker ps |grep -i $i|awk '{print $1}')
   done
   ```
4. Check that all services recover (all replicas should be 1/1 after enough time ~5 minutes):
   ```bash
   docker service ls
   ```
5. Check that OSM is working again:
   ```bash
   osm vnfpkg-list
   osm vim-list
   ```

In case of installation based on K8s, the command to kill abruptly all containers (step 3) should be:

```bash
for i in {k8s_kafka,k8s_keystone,k8s_mysql,k8s_lcm,k8s_light-ui,k8s_mon_,k8s_mongo,k8s_nbi,k8s_pol,k8s_prometheus,k8s_ro,k8s_zookeeper}; do
  docker rm -f $(docker ps |grep -i $i|awk '{print $1}')
done
```

## Testing of optional or environment-dependent features

### \[OPTIONAL-01\] Support of multi-segment VIM-managed networks

For this to be tested, we need a specific VIM with multi-segment support and appropriate configuration with a companion SDN. In the case of Openstack, if it supports L2GW, the following config options need to be used:

```json
multisegment_support: true, l2gw_support: true, multisegment_vlan_range: ["200-500"]
```

Then, any test that deploys VNFs with SR-IOV interfaces and VNFs with PARAVIRT interfaces in the same network will be enough for testing. For instance, the NS `hackfest_multisegment_ns` has been created using `hackfest_epasriov_vnf` and `hackfest_cloudinit_vnf` VNF:

```bash
osm nfpkg-create hackfest_epasriov_vnf.tar.gz
osm nspkg-create hackfest_multisegment_ns.tar.gz
osm ns-create --ns_name hackfest-multisegment --nsd_name hackfest_epasriov-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID>
osm ns-list
osm ns-show hackfest-multisegment
```

Once it is up and running, check in your VIM that the network has two segments. For instance, in Openstack, run the following command:

```bash
openstack network show <network_id>
# There should be two segments, one based on VxLAN and another based on VLAN with an id in the range specified in the config dictionary.
```

Then, try to ping from each VM to the other

```bash
# Get the management IP address of the VNF in this way:
MGMT_IP_VNF1=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf1" {print $14}'`
MGMT_IP_VNF2=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf2" {print $14}'`
MGMT_IP_VNF3=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf3" {print $14}'`
# Then get the IP of the dataVM for each VNF in the following way
VNF1_ID=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf1" {print $2}'`
osm vnf-show VNF1_ID --literal |grep eth0|grep internal
dataVM_IP_VNF1=`osm vnf-show VNF1_ID --literal |grep eth0|grep internal|awk '{print $3}' |awk -F "," '{print $1}'
VNF2_ID=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf2" {print $2}'`
osm vnf-show VNF2_ID --literal |grep eth0|grep internal
dataVM_IP_VNF2=`osm vnf-show VNF2_ID --literal |grep eth0|grep internal|awk '{print $3}' |awk -F "," '{print $1}'
VNF3_ID=`osm vnf-list --ns hackfest-multisegment |awk '$8=="vnf3" {print $2}'`
osm vnf-show VNF3_ID --literal |grep eth0|grep internal
dataVM_IP_VNF3=`osm vnf-show VNF3_ID --literal |grep eth0|grep internal|awk '{print $3}' |awk -F "," '{print $1}'
# Connect to the mgmt VM of VNF 1 (the password to the default user has been injected via cloud-init)
ssh ubuntu@<MGMT_IP_VNF1> (pwd: osm4u)
# Jump to the dataVM
ssh -i test4.pem ubuntu@<dataVM_IP_VNF1>
# Ping to the other VMs
ping <dataVM_IP_VNF2>
ping <dataVM_IP_VNF3>
# Repeat the same in the othe VMs
```

Finally, clean your environment:

```bash
osm ns-delete hackfest-multisegment
```

### \[OPTIONAL-02\] VIM-driven SFC (service chaining)

For this test, Openstack must be configured to allow SFC.

1. Onboard SFC descriptors

   ```bash
   osm vnfpkg-create vnf_nsh_vnf.tar.gz
   osm vnfpkg-create endpoint_vnf.tar.gz
   osm nspkg-create sfc_nsh_ns.tar.gz
   ```

2. Deploy the scenario

   ```bash
   osm ns-create --ns_name SFC-test --nsd_name sfc_nsh_nsd --ssh_keys <PUBLIC-KEY> --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
   osm ns-list
   ```

3. To test that the SFC works, we will listen in port 80 of server with `10.10.10.12`, we will send traffic from server
   `10.10.10.11` to the `10.10.10.12:80`, and we will monitor traffic in `vnf_nsh_vnfd`

   ```bash
   osm vnf-list --ns SFC-test
   ssh -i <PRIVATE-KEY> ubuntu@<MGMT-IP of 10.10.10.12>
   > # Enable the data interface in the OS to use dhcp. It will get 10.10.10.12
   > nc -l 80
   ssh -i <PRIVATE-KEY> ubuntu@<MGMT-IP of 10.10.10.11>
   > # Enable the data interface in the OS to use dhcp. It will get 10.10.10.11
   > nc 10.10.10.12 80
   ssh -i <PRIVATE-KEY> root@<MGMT-IP of vnf_nsh_vnfd> (pwd: opnfv)
   > tcpdump -i <Interface with 10.10.10.X network>
   ```

### \[OPTIONAL-03\] Alternate images depending on VIM

```diff
-This test description is ported as-is from the old test plan and might require further revision.
-In case of finding any issue during test execution, please report to OSM_TECH@LIST.ETSI.ORG.
```

1. Onboard the packages:
   ```bash
   osm vnfd-create hackfest_basic_vnf.tar.gz
   osm nsd-create hackfest_basic_ns.tar.gz
   ```
2. Deploy in any VIM:
   ```bash
   osm ns-create --ns_name hackfestVIM1 --nsd_name hackfest_basic-ns --vim_account    <VIM1_ACCOUNT_NAME>|<VIM1_ACCOUNT_ID> --ssh_keys
   <KEY1_PUBKEY_FILE> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM1_MGMT_NAME>} ] }'
   osm ns-list
   osm ns-show hackfestVIM1
   ssh -i <KEY1_PRIVKEY_FILE> ubuntu@<MGMT_IP_VIM1>
   ```
3. Deploy in an AWS VIM (the VNF package indicates an alternative image for it):
   ```bash
   osm ns-create --ns_name hackfestVIM2 --nsd_name hackfest_basic-ns --vim_account    <VIM2_ACCOUNT_NAME>|<VIM2_ACCOUNT_ID> --ssh_keys
   <KEY1_PUBKEY_FILE> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM2_MGMT_NAME>} ] }'
   osm ns-list
   osm ns-show hackfestVIM2
   ssh -i <KEY1_PRIVKEY_FILE> ubuntu@<MGMT_IP_VIM2>
   osm ns-delete hackfestVIM1
   osm ns-delete hackfestVIM2
   ```

## References

- \[1\]: [OSM Information Model](https://osm.etsi.org/wikipub/index.php/OSM_Information_Model)
- \[2\]: [PROX Documentation](https://01.org/intel-data-plane-performance-demonstrators/documentation/prox-documentation)
