# Test plan for validation of new Release EIGHT features

## Quotas per OSM project

### \[QUOTAS-01\] Quota enforcement

```diff
-This test description might need to be split into several tests (\[QUOTAS-0n\]) to ease the tracking.
```

This test will check project creation edition and isolation. It tests the quotas per project. Finally it tests that projects cannot be deleted with dependencies.

1. Create project with quotas. Tests `osmclient` with both comma-separated list and repetition of `--quotas` option:
   ```bash
   osm project-create p3 --quotas vnfds=1,nsds=1 --quotas ns_instances=1
   osm project-show p3 | grep -q -e '"vnfds": 1' || echo "FAIL"
   osm project-show p3 | grep -q -e '"nsds": 1' || echo "FAIL"
   osm project-show p3 | grep -q -e '"ns_instances": 1' || echo "FAIL"

   osm project-create p2 --quotas vnfds=1,nsds=1,ns_instances=1
   osm project-show p2 | grep -q -e '"vnfds": 1' || echo "FAIL"
   osm project-show p2 | grep -q -e '"nsds": 1' || echo "FAIL"
   osm project-show p2 | grep -q -e '"ns_instances": 1' || echo "FAIL"
   ```
2. Create user and assign `project_admin` role for both projects:
   ```bash
   osm user-create u --password u --project-role-mappings p3,project_admin --project-role-mappings p2,project_admin
   ```
3. Change project name:
   ```bash
   osm project-update p3 --name p1
   osm project-show p1 | grep -q -e '"vnfds": 1' || echo "FAIL"
   osm project-show p1 | grep -q -e '"nsds": 1' || echo "FAIL"
   osm project-show p1 | grep -q -e '"ns_instances": 1' || echo "FAIL"
   osm user-show u | grep -q -e '"p3"' && echo "FAIL"   # user uses renamed project 'p1', not 'p3'
   ```
4. List packages (list must be empty) without any uuid. Using regular expression for grep:
   ```bash
   osm --project p1 --password u --user u vnfpkg-list | grep -q -E "[0-9a-f]{8}-[0-9a-f]{4}-[0-5][0-9a-f]{3}-[089ab][0-9a-f]{3}-[0-9a-f]{12}" && echo "FAIL"
   ```
5. Add packages
   ```bash
   osm --project p1 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=v1;name=v1" || echo "FAIL"
   ```
6. Force a failure for exceeded quota:
   ```bash
   osm --project p1 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=v2;name=v2" && echo "FAIL"
   ```
7. Enlarge quota:
   ```bash
   osm project-update p1 --quotas "vnfds=10,nsds=,ns_instances="
   osm project-show p1 | grep -q -e '"vnfds": 10' || echo "FAIL"
   osm project-show p1 | grep -q -e '"nsds":' && echo "FAIL"
   ```
8. Add more packages:
   ```bash
   for i in {2..10} ; do
     osm --project p1 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=v${i};name=v${i}" || echo "FAIL"
   done
   ```
9. Another `vnfpkg` created should fail for quota limits:
   ```bash
   osm --project p1 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=v11;name=v11" && echo "FAIL"
   ```
10. Switch to project 2
    ```bash
    osm --project p2 --password u --user u vnfpkg-list | grep -q -E "[0-9a-f]{8}-[0-9a-f]{4}-[0-5][0-9a-f]{3}-[089ab][0-9a-f]{3}-[0-9a-f]{12}" && echo "FAIL"
    ```
11. Test quota in project 2. Only one package is allowed
    ```bash
    osm --project p2 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=vp2_1;name=vp2_1"
    osm --project p2 --password u --user u vnfpkg-create hackfest_basic_vnf.tar.gz --override "id=vp2_2;name=vp2_2" && echo "FAIL"
    ```
12. Check that one project cannot see the scope of other projects:
    ```bash
    osm --project p2 --password u --user u --all-projects vnfpkg-list && echo "FAIL"  # not authorized
    osm --project p1 --password u --user u vnfpkg-show vp2_1  && echo "FAIL"
    osm --project p2 --password u --user u vnfpkg-show v1  && echo "FAIL"
    ```
13. Delete. Test dependencies:
    1. Remove packages
       ```bash
       for i in {1..10} ; do
         osm --project p1 --password u --user u vnfpkg-delete v${i}
       done
       osm --project p2 --password u --user u vnfpkg-delete vp2_1
       ```
    2. Delete project will fail because in use by user 'u'
       ```bash
       osm project-delete p2 && echo "FAIL"

       osm user-update u --remove-project p2  # remove project from user
       osm user-show u | grep -q -e '"p2"' && echo "FAIL"
       osm project-delete p2 || echo "FAIL"  # p2 can be deleted
       osm project-delete p1 && echo "FAIL"  # p1 still in use by user 'u'

       osm user-delete u
       osm project-delete p1

       osm project-list | grep -e p1 -e p2 && echo "FAIL"
       ```

## Production readiness

### \[PR-01\] HA VCA

First, check the number of nodes in your nodes in your VCA with the following command:

```bash
$ juju status -m controller
Model       Controller  Cloud/Region         Version  SLA          Timestamp
controller  osm         localhost/localhost  2.7.7    unsupported  14:45:51Z

Machine  State    DNS           Inst id        Series  AZ  Message
0        started  10.16.13.145  juju-ff4c51-0  xenial      Running
```

Then, in the MongoDB container, enter to a `mongo` shell, and execute the following:

```
> use osm
switched to db osm
> db.admin.find({"_id": "juju"})
{ "_id" : "juju", "api_endpoints" : [ "10.16.13.145:17070" ] }
```

You should see only 1 api_endpoint in the list, and should match the one in the command `juju status -m controller` output.

At this point, you can go to [this test](#basic-06-vnf-with-charm) to deploy a proxy charm.

Now we will scale the juju controller.

```bash
$ juju enable-ha
maintaining machines: 0
adding machines: 1, 2
```

It takes a bit to have the cluster set. Do `watch -c juju status -m controller --color` to watch the process.

When you see the following output, it means the machines are ready, but it takes a couple of minutes for the cluster to be properly set.

```
Model       Controller  Cloud/Region         Version  SLA          Timestamp
controller  osm         localhost/localhost  2.7.7    unsupported  16:24:48Z

Machine  State    DNS           Inst id        Series  AZ  Message
0        started  10.16.13.145  juju-ff4c51-0  xenial      Running
1        started  10.16.13.252  juju-ff4c51-1  xenial      Running
2        started  10.16.13.220  juju-ff4c51-2  xenial      Running
```

Now, we should go to the `mongo` shell again to check the list of endpoints:
> Note: It can take up to 5 minutes to see the endpoints.

```
> use osm
switched to db osm
> db.admin.find({"_id": "juju"})
{ "_id" : "juju", "api_endpoints" : [ "10.16.13.145:17070", "10.16.13.252:17070", "10.16.13.220:17070" ] }
```

When we deployed the proxy charm, only the first node existed. So now, we are going to stop that container (juju-ff4c51-0), and remove the network service previously created

```
$ lxc stop juju-ff4c51-0
$ juju status -m controller
Model       Controller  Cloud/Region         Version  SLA          Timestamp
controller  osm         localhost/localhost  2.7.7    unsupported  16:32:47Z

Machine  State    DNS           Inst id        Series  AZ  Message
0        down     10.16.13.145  juju-ff4c51-0  xenial      Running
1        started  10.16.13.252  juju-ff4c51-1  xenial      Running
2        started  10.16.13.220  juju-ff4c51-2  xenial      Running
```

Now I will remove the created network service:

```
$ osm ns-list
+------------------+--------------------------------------+---------------------+----------+-------------------+---------------+
| ns instance name | id                                   | date                | ns state | current operation | error details |
+------------------+--------------------------------------+---------------------+----------+-------------------+---------------+
| hackfest5        | 779f984b-0448-4ee6-b787-e45caa6e35c4 | 2020-07-01T15:40:50 | READY    | IDLE (None)       | N/A           |
+------------------+--------------------------------------+---------------------+----------+-------------------+---------------+
To get the history of all operations over a NS, run "osm ns-op-list NS_ID"
For more details on the current operation, run "osm ns-op-show OPERATION_ID"
$ osm ns-delete hackfest5
Deletion in progress
$ osm ns-list
+------------------+----+------+----------+-------------------+---------------+
| ns instance name | id | date | ns state | current operation | error details |
+------------------+----+------+----------+-------------------+---------------+
+------------------+----+------+----------+-------------------+---------------+
To get the history of all operations over a NS, run "osm ns-op-list NS_ID"
For more details on the current operation, run "osm ns-op-show OPERATION_ID"
```

The network service should be removed fine, without leaving models behind.

### \[PR-02\] HA Proxy Charms

These are the packages needed for this test:

- [VNFD Package](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ha_proxy_charm_vnf)
- [NSD Package](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages/tree/master/charm-packages/ha_proxy_charm_ns)

First, create a `params.yaml` file that will force 2 units of the `vnf-member-index=1` Proxy Charm:

```yaml
additionalParamsForVnf:
  - member-vnf-index: "1"
    config-units: 2
```

```bash
$ osm ns-create --ns_name ha-proxy-charms --nsd_name  ha_proxy_charm-ns --vim_account openstack --config_file params.yaml
7a8b8fb1-3cea-4e86-8cf3-178771d9ac3a
$ juju switch 7a8b8fb1-3cea-4e86-8cf3-178771d9ac3a
controller:admin/default -> controller:admin/7a8b8fb1-3cea-4e86-8cf3-178771d9ac3a
$ watch -c juju status --color
Model                                 Controller  Cloud/Region       Version  SLA          Timestamp
7a8b8fb1-3cea-4e86-8cf3-178771d9ac3a  controller  lxd-cloud/default  2.8.0    unsupported  12:42:29+02:00

App                   Version  Status  Scale  Charm            Store  Rev  OS      Notes
app-vnf-3922753a5d75           active      2  simple-ha-proxy  local    0  ubuntu
app-vnf-feba90bf85a7           active      1  simple-ha-proxy  local    1  ubuntu

Unit                     Workload  Agent  Machine  Public address  Ports  Message
app-vnf-3922753a5d75/0*  active    idle   0        10.54.156.223
app-vnf-3922753a5d75/1   active    idle   2        10.54.156.29
app-vnf-feba90bf85a7/0*  active    idle   1        10.54.156.128

Machine  State    DNS            Inst id        Series  AZ  Message
0        started  10.54.156.223  juju-cf61c0-0  xenial      Running
1        started  10.54.156.128  juju-cf61c0-1  xenial      Running
2        started  10.54.156.29   juju-cf61c0-2  xenial      Running
```

We can see there are two units for the Proxy Charm of the vnf-member-index=1. The units that have `*` are the leaders. Now we will remove leader machine of the scaled Proxy Charm.

```bash
$ juju remove-machine 0 --force
$ juju status
Model                                 Controller  Cloud/Region       Version  SLA          Timestamp
7a8b8fb1-3cea-4e86-8cf3-178771d9ac3a  controller  lxd-cloud/default  2.8.0    unsupported  12:48:27+02:00

App                   Version  Status  Scale  Charm            Store  Rev  OS      Notes
app-vnf-3922753a5d75           active      1  simple-ha-proxy  local    0  ubuntu
app-vnf-feba90bf85a7           active      1  simple-ha-proxy  local    1  ubuntu

Unit                     Workload  Agent  Machine  Public address  Ports  Message
app-vnf-3922753a5d75/1*  active    idle   2        10.54.156.29
app-vnf-feba90bf85a7/0*  active    idle   1        10.54.156.128

Machine  State    DNS            Inst id        Series  AZ  Message
1        started  10.54.156.128  juju-cf61c0-1  xenial      Running
2        started  10.54.156.29   juju-cf61c0-2  xenial      Running
```

Now the second unit is the leader. Let's execute an action:

```bash
$ osm ns-action ha-proxy-charms --vnf_name 1 --action_name touch --params '{filename: /home/ubuntu/mysecondunit}'
c4a32267-8d62-4751-9c0f-8c0f65b9ccc7
$ osm ns-op-list ha-proxy-charms
+--------------------------------------+-------------+-------------+-----------+---------------------+--------+
| id                                   | operation   | action_name | status    | date                | detail |
+--------------------------------------+-------------+-------------+-----------+---------------------+--------+
| 3bc998b2-f120-485b-bbfd-6965e800b960 | instantiate | N/A         | COMPLETED | 2020-07-13T12:41:10 | -      |
| c4a32267-8d62-4751-9c0f-8c0f65b9ccc7 | action      | touch       | COMPLETED | 2020-07-13T12:49:34 | -      |
+--------------------------------------+-------------+-------------+-----------+---------------------+--------+
```

The action has been successfully executed.

## Kubernetes Features

### [K8S-05] Kubernetes Proxy Charms

This feature is only supported in the Charmed Installation and the Kubernetes Installation. Therefore make sure you have one of either installed.

1. Clone the osm-packages repository and download the submodules:

```bash
git clone https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages.git
cd osm-packages
git submodule update --init
```

2. Build the hackfest_k8sproxycharm Packages and upload them:

```bash
tar -czf hackfest_k8sproxycharm_vnf.tar.gz hackfest_k8sproxycharm_vnf/
tar -czf hackfest_k8sproxycharm_ns.tar.gz hackfest_k8sproxycharm_ns/
osm upload-package hackfest_k8sproxycharm_vnf.tar.gz
osm upload-package hackfest_k8sproxycharm_ns.tar.gz
```

3. Create the Network Service:

```bash
osm ns-create --ns_name hf-k8s --nsd_name hackfest_k8sproxycharm-ns --vim_account openstack --config '{vld: [ {name: mgmtnet, vim-network-name: osm-ext} ] }'
```

4. Wait until the Network Service is in the READY State:

```bash
osm ns-list
```

5. Once the Network Service is ready execute the following action:

```bash
osm ns-action hf-k8s --vnf_name 1 --action_name touch --params '{filename: /home/ubuntu/mytouch1}'
```

6. Wait until the action is in COMPLETED state:

```bash
osm ns-op-list hf-k8s
```

7. SSH into VNF 1  and make sure that the `firsttouch` and `mytouch1` files have been created.

## Automatic Placement Feature (feature 7953)

### [PLA-01] Basic Placement

_Note. The following is a basic test of PLA using a nsd with a single vnf and only two VIMs. It does therefore not capture more elaborate capabilities of the PLA module such as e.g. pinning and vld constraints._

1. Install PLA 
   - PLA is an optional module, install by adding --pla to install script
   ```bash
   $ ./install.sh <...> --pla`
   ```

2. Create the VIMs
   - Create at least two VIM instances
     ```bash
     $ osm vim-create --name <VIM_NAME> --account_type openstack --auth_url <URL> --user <USER> --password <PASSWORD> -- tenant <TENANT> --description "my description"`
     ```
   - The list of VIMs should be similar to e.g.:
     ```text
     +------------+--------------------------------------+
     | vim name   | uuid                                 |
     +------------+--------------------------------------+
     | OpenStack2 | eb553051-5b6c-4ad6-939b-2ad23bd82e57 |
     | OpenStack1 | 576bbe0a-b95d-4ced-a63e-f387f8e6e2ce |
     +------------+--------------------------------------+
     ```

3. Get the descriptors and onboard them
   ```bash
   $ wget http://osm-download.etsi.org/ftp/osm-6.0-six/8th-hackfest/packages/hackfest-basic_vnfd.tar.gz
   $ wget http://osm-download.etsi.org/ftp/osm-6.0-six/8th-hackfest/packages/hackfest-basic_nsd.tar.gz
   $ osm vnfd-create hackfest-basic_vnfd.tar.gz
   $ osm nsd-create hackfest-basic_nsd.tar.gz
   ```

4. Create the price lists for compute and transport links
   - Create the price list files `vnf_price_list.yaml` and `pil_price_list.yaml`. `vnf_price_list.yaml` should contain an entry for the vnfd to use, e.g.
     ```yaml
     - vnfd: hackfest-basic_vnfd
       prices:
         - vim_url: http://10.234.12.47:5000/v3
           vim_name: OpenStack1
           price: 5
         - vim_url: http://10.234.12.44:5000/v3
           vim_name: OpenStack2
           price: 10
      ```
      - Note `vim_url` and `vim_name` corresponding to step 2.
   - `pil_price_list.yaml` describes the inter-vim communication links, e.g.
     ```yaml
     pil:
       - pil_description: Link between OpenStack1 and OpenStack2
         pil_price: 5
         pil_latency: 20
         pil_endpoints:
           - OpenStack1
           - OpenStack2
     ```
     - Note `pil_endpoints` using VIM names corresponding to step 2.

5. Update the PLA container with price list and inventory file
   ```bash
   $ docker cp vnf_price_list.yaml $(docker ps -qf name=osm_pla):/placement/.
   $ docker cp pil_price_list.yaml $(docker ps -qf name=osm_pla):/placement/.
   ```

6. Instantiate the service using the placement engine
   - Use the more expensive vim for `--vim_account`.
     ```bash
     $ osm ns-create --ns_name pla_test --nsd_name hackfest-basic_nsd --vim_account OpenStack2 --config '{placement-engine: PLA}'
     ```

7. Check VNF deployment
   - The VNF will be deployed to a VIM according to the content of price list.
     ```bash
     $ osm vnf-list
     ```

     ```text
     +--------------------------------------+------+--------------------------------------+------------------+----------------------+--------------------------------------     +----------------+
     | vnf id                               | name | ns id                                | vnf member index | vnfd name            | vim account id                       | ip address     |
     +--------------------------------------+------+--------------------------------------+------------------+----------------------+--------------------------------------+----------------+
     | cbacb383-c2e9-4fda-b066-649165e294bc | -    | 54d36395-53cd-4b99-817b-5900f88523da | 1                | hackfest-basic_vnfd  | 576bbe0a-b95d-4ced-a63e-f387f8e6e2ce | 192.168.241.9  |
     +--------------------------------------+------+--------------------------------------+------------------+----------------------+--------------------------------------+----------------+
     ```

In this example the VNF is deployed to OpenStack1 being the cheapest VIM for the VNF.

## Service Assurance

### \[SA-08\] VNF Indicator collection using Prometheus exporters

As a precondition for this test, please make sure:

1. Your VIM has an accessible management network (in this case called "<VIM_MGMT_NET>")
2. Your VIM has the appropriate test image, in this case `vyos-1.1.7` ([vyos-1.1.7-cloudinit.qcow2](https://osm-download.etsi.org/ftp/images/tests/vyos-1.1.7-cloudinit.qcow2)).
3. UDP port 161 (SNMP) is reachable from outside. Typically, you would need to authorize this port in the default security group of your VIM.
4. You have an OSM installation over K8s (e.g. installed using the `-c k8s` switch). This feature is not available in OSM installations based on Docker Swarm.

These are the steps:

1. Onboard the packages ([refer to OSM's Git repo for packages](https://osm.etsi.org/gitlab/vnf-onboarding/osm-packages)):
   ```bash
   osm vnfpkg-create snmp_ee_vnf
   osm nspkg-create snmp_ee_ns
   ```
2. Launch the NS:
   ```bash
   osm ns-create --ns_name snmpee_sa08 --nsd_name snmp_ee-ns --vim_account <VIM_ACCOUNT_NAME>|<VIM_ACCOUNT_ID> --config '{vld: [ {name: mgmtnet, vim-network-name: <VIM_MGMT_NET>} ] }'
   osm ns-list
   ```
3. Wait ~5 minutes for the SNMP metrics to get collected.
4. After that, check some of them at Prometheus, visiting [`http://[OSM_IP]:9091`](http://[OSM_IP]:9091) and looking for `ifMtu` and `ifInOctets` (you should see one value per VM interface).
5. Finally, delete the NS and the packages:
   ```bash
   osm ns-delete snmpee_sa08
   osm nsd-delete snmp_ee-ns
   osm vnfd-delete snmp_ee-vnf
   ```
   